SPOOL .\Logs\10-Package_Productos.log
PROMPT Creando Package Productos... 

/************************************************************************************************************************
* Nombre......: PK_PRODUCTOS                                                                                            *
* Descripcin.: Agrupa la gestin de productos                                                                          *
*************************************************************************************************************************/
 
CREATE OR REPLACE PACKAGE UOC.PK_PRODUCTOS AS

PROCEDURE usp_insertar_producto (

  I_Producto       IN Producto.N_Producto%Type,
  I_Descripcion    IN Producto.V_Descripcion%Type,
  I_FIncorporacion IN Producto.F_Incorporacion%Type,
  O_RSP            OUT NOCOPY VARCHAR2

  );

PROCEDURE usp_actualizar_producto (

  I_Producto       IN Producto.N_Producto%Type,
  I_Descripcion    IN Producto.V_Descripcion%Type,
  I_FIncorporacion IN Producto.F_Incorporacion%Type,
  O_RSP            OUT NOCOPY VARCHAR2

  );
  
PROCEDURE usp_eliminar_producto (

  I_Producto IN Producto.N_Producto%Type,
  O_RSP      OUT NOCOPY VARCHAR2

  );

END PK_PRODUCTOS;

/

CREATE OR REPLACE PACKAGE BODY UOC.PK_PRODUCTOS AS

/****************************************************************************************************************************************
*                                                                                                                                       * 
* Nombre........: usp_insertar_producto                                                                                                 *
* Descripcin...: Permite aadir un nuevo producto                                                                                      *
*                                                                                                                                       *
* Parmetros....: INPUT..: I_Producto       (Cdigo de identificin del producto      ) - NUMBER(13)        - Obligatorio               *
*                          I_Descripcion    (Descripcin del producto                 ) - VARCHAR2(50 CHAR) - Obligatorio               *
*                          I_FIncorporacion (Fecha en la que se incorpora al catlogo ) - DATE              - Obligatorio               *
*                 OUTPUT.: O_RSP            (Resultado de la ejecucin                ) - VARCHAR2                                      *
*                                                                                                                                       *
* Precondicin..: No existe el producto.                                                                                                *
* Postcondicin.: Se incorpora el nuevo producto a la tabla.                                                                            *
*                                                                                                                                       *
* Resultado de la ejecucin:                                                                                                            *
*                                                                                                                                       *
*                 OK - El alta se realiz correctamente.                                                                                *
*                 ERROR: No se informa el cdigo de producto.                                                                           *
*                 ERROR: El cdigo de producto informado no es correcto.                                                                *
*                 ERROR: No se informa la descripcin del producto.                                                                     *
*                 ERROR: No se informa la fecha de incorporacion al catlogo.                                                           *
*                 ERROR: La fecha de incorporacin informada no es correcta.                                                            *
*                 ERROR: El producto ya existe.                                                                                         *
*                 ERROR: El cdigo de producto informado no es correcto.                                                                *
*                 ERROR: SQLCODE - SQLERRM                                                                                              *
*                                                                                                                                       *
*****************************************************************************************************************************************/

PROCEDURE usp_insertar_producto (

  I_Producto         IN Producto.N_Producto%Type,
  I_Descripcion      IN Producto.V_Descripcion%Type,
  I_FIncorporacion   IN Producto.F_Incorporacion%Type,
  O_RSP              OUT NOCOPY VARCHAR2

  )

IS

  Procedimiento      VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);
  
  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);
	
  Ex_Descripcion     EXCEPTION;
  Ex_FIncorporacion  EXCEPTION;

BEGIN
	
  -- Inicializacin de variables
  
  Procedimiento       := 'PK_PRODUCTOS.usp_insertar_producto';
  Parametros_entrada  := '(' || I_Producto || ', ' || I_Descripcion  || ', ' || I_FIncorporacion || ')';
  O_RSP               := 'OK';
  
  -- Verificacin de datos
  
  IF (I_Descripcion IS NULL or TRIM(I_Descripcion) IS NULL) THEN
    RAISE Ex_Descripcion;
  END IF;
  
  PK_VERIFICACION.usp_verificar_producto(I_Producto, 'A', O_RSP);

  IF (I_FIncorporacion IS NULL) THEN
	  RAISE Ex_FIncorporacion;
  END IF;
  
  IF (O_RSP = 'OK') THEN
	IF (PK_FUNCIONES.f_es_fecha(TO_CHAR(I_FIncorporacion,'dd/mm/yyyy')) = FALSE) THEN
	  RAISE Ex_FIncorporacion;
	END IF;
  END IF;
  
  IF (O_RSP = 'OK') THEN
  
    -- Alta del registro
    
    INSERT INTO PRODUCTO	 
      (N_PRODUCTO, V_DESCRIPCION, F_INCORPORACION, F_ALTA)
      VALUES (I_Producto, I_Descripcion, I_FIncorporacion, SYSTIMESTAMP);
  
  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
  
  -- Excepciones 
   
  EXCEPTION

    WHEN Ex_Descripcion THEN
      O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_insertar_producto',1));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);				
	  
	WHEN Ex_FIncorporacion THEN
      O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_insertar_producto',2));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);				
  
    WHEN DUP_VAL_ON_INDEX THEN
      O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_insertar_producto',3));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);			
          
    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
        
END usp_insertar_producto;

/****************************************************************************************************************************************
*                                                                                                                                       *
* Nombre........: usp_actualizar_producto                                                                                               *
* Descripcin...: Permite actualizar la informacin de una producto.                                                                    *
*                 Los campos opcionales puede estar informados o ser NULL                                                               *
*                                                                                                                                       *
* Parmetros....: INPUT..: I_Producto       (Cdigo de identificin del producto      ) - NUMBER(13)        - Obligatorio               *
*                          I_Descripcion    (Descripcin del producto                 ) - VARCHAR2(50 CHAR) - Opcional                  *
*                          I_FIncorporacion (Fecha en la que se incorpora al catlogo ) - DATE              - Opcional                  *
*                 OUTPUT.: O_RSP            (Resultado de la ejecucin                ) - VARCHAR2                                      *
*                                                                                                                                       *
* Precondicin..: Existe el producto a modificar.                                                                                       *
* Postcondicin.: Se actualizan los datos correspondientes al producto.                                                                 *
*                                                                                                                                       *
* Resultado de la ejecucin:                                                                                                            *
*                                                                                                                                       *
*                 OK - La modificacin se realiz correctamente.                                                                        *
*                 ERROR: No se informa el cdigo de producto.                                                                           *
*                 ERROR: No existe el producto a modificar.                                                                             *
*                 ERROR: La fecha de incorporacin informada no es correcta.                                                            *
*                 ERROR: SQLCODE - SQLERRM                                                                                              *
*                                                                                                                                       *
*****************************************************************************************************************************************/

PROCEDURE usp_actualizar_producto (

  I_Producto         IN Producto.N_Producto%Type,
  I_Descripcion      IN Producto.V_Descripcion%Type,
  I_FIncorporacion   IN Producto.F_Incorporacion%Type,
  O_RSP              OUT NOCOPY VARCHAR2

  )

IS
  
  Procedimiento      VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);
  
  Registros          NUMBER;
  
  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);
	
  Ex_Producto        EXCEPTION;
  Ex_FIncorporacion  EXCEPTION;
  Ex_Registros       EXCEPTION;

  -- Guardan los valores previos a la actualizacin
  
  TMP_Descripcion      Producto.V_Descripcion%Type;
  TMP_FIncorporacion   Producto.F_Incorporacion%Type;
  
BEGIN
	
  -- Inicializacin de variables
  
  Procedimiento       := 'PK_PRODUCTOS.usp_actualizar_producto';
  Parametros_entrada  := '(' || I_Producto || ', ' || I_Descripcion  || ', ' || I_FIncorporacion || ')';
  O_RSP               := 'OK';
  
  -- Verifica que est informado el cdigo de identificacin del producto
  
  IF (I_Producto IS NULL) THEN
    RAISE Ex_Producto;
  END IF;
  
  -- Verifica la existencia y en caso afirmativo recupera la informacin del producto
  
  SELECT COUNT(*) INTO Registros FROM PRODUCTO
     WHERE N_PRODUCTO = I_Producto;

  IF (Registros = 0) THEN
	RAISE Ex_Registros;
  END IF;
  
  SELECT V_Descripcion, F_Incorporacion INTO
    TMP_Descripcion, TMP_FIncorporacion
  FROM PRODUCTO
  WHERE N_PRODUCTO = I_Producto;
  
  -- Verificacin de los datos recibidos
  
  IF (I_Descripcion IS NOT NULL or TRIM(I_Descripcion) IS NOT NULL)  THEN
	TMP_Descripcion := I_Descripcion;
  END IF;	
  
  IF (O_RSP = 'OK' and I_FIncorporacion IS NOT NULL) THEN
	
	IF (PK_FUNCIONES.f_es_fecha(TO_CHAR(I_FIncorporacion,'dd/mm/yyyy')) = FALSE) THEN
	  RAISE Ex_FIncorporacion;
	ELSE
	  TMP_FIncorporacion := I_FIncorporacion;
	END IF;
  
  END IF;	
  
  IF (O_RSP = 'OK') THEN
  
    -- Actualizacin del registro (si est eliminado lgicamente lo activa)
    
    UPDATE PRODUCTO SET 
	  V_DESCRIPCION    = TMP_Descripcion,
	  F_INCORPORACION  = TMP_FIncorporacion,
      F_MODIFICACION   = SYSTIMESTAMP,
      F_ELIMINACION    = NULL
      WHERE N_PRODUCTO = I_Producto;
  
    IF SQL%NOTFOUND THEN
      RAISE NO_DATA_FOUND;
    END IF;
    
  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
  
  -- Excepciones 
  
  EXCEPTION

    WHEN NO_DATA_FOUND THEN
      O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_actualizar_producto',1));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);				

    WHEN Ex_Producto THEN
      O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_actualizar_producto',2));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);				
	  
	WHEN Ex_FIncorporacion THEN
      O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_actualizar_producto',3));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);				
	 
	WHEN Ex_Registros THEN
	  O_RSP := TRIM(PK_FUNCIONES.f_mensaje('PK_PRODUCTOS.usp_actualizar_producto',4));
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);				
	  
    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
        
END usp_actualizar_producto;

/****************************************************************************************************************************************
*                                                                                                                                       *
* Nombre........: usp_eliminar_producto.                                                                                                *
* Descripcin...: Permite eliminar un producto.                                                                                         *
*                                                                                                                                       *
* Parmetros....: INPUT..: I_Producto (Cdigo de identificin del producto) - NUMBER(13) - Obligatorio                                  *
*                 OUTPUT.: O_RSP      (Resultado de la ejecucin          ) - VARCHAR2                                                  *
*                                                                                                                                       *
* Precondicin..: Existe el producto a eliminar.                                                                                        *
* Postcondicin.: Se realiza la eliminacin fsica o lgica del producto.                                                               *
*                                                                                                                                       *
* Resultado de la ejecucin:                                                                                                            *
*                                                                                                                                       *
*                 OK - La eliminacin se realiz correctamente.                                                                         *
*                 ERROR: No se informa el cdigo de producto.                                                                           *
*                 ERROR: El producto informado no existe.                                                                               *
*                 ERROR: El producto informado est anulado                                                                             *
*                 ERROR: El cdigo de producto informado no es correcto.                                                                *
*                 ERROR: SQLCODE - SQLERRM                                                                                              *
*                                                                                                                                       *
*****************************************************************************************************************************************/

PROCEDURE usp_eliminar_producto (

  I_Producto IN Producto.N_Producto%Type,
  O_RSP      OUT NOCOPY VARCHAR2

  )

IS

  Procedimiento      VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);
  
  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);
	
BEGIN
	
  -- Inicializacin de variables
  
  Procedimiento       := 'PK_PRODUCTOS.usp_eliminar_producto';
  Parametros_entrada  := '(' || I_Producto || ')';
  O_RSP               := 'OK';
  
  -- Verificacin de datos
  
  PK_VERIFICACION.usp_verificar_producto(I_Producto, 'E', O_RSP);
  
  IF (O_RSP = 'OK') then
  
    -- Elimina el producto fsica o lgicamente.
  
    BEGIN
  
      DELETE FROM PRODUCTO WHERE N_PRODUCTO = I_Producto;
    
      EXCEPTION
  
        WHEN OTHERS THEN
    
          BEGIN
          
            UPDATE PRODUCTO
            SET    F_ELIMINACION = SYSTIMESTAMP
            WHERE  N_PRODUCTO = I_Producto;
          
          EXCEPTION
          
            WHEN OTHERS THEN
              Err_Codigo      := SQLCODE;
              Err_Descripcion := SQLERRM;
      
              O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                           || '-' 
                                           || Err_Descripcion;
      
              PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
              
          END;
    
    END;
    
  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
  
  -- Excepciones 
  
  EXCEPTION

    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Procedimiento, Parametros_entrada, O_RSP);
        
END usp_eliminar_producto;

END PK_PRODUCTOS;

/

SPOOL OFF;
