SPOOL .\Logs\12-Package_Consultas.log
PROMPT Creando Package Consultas... 

/************************************************************************************************************************
* Nombre......: PK_CONSULTAS                                                                                            *
* Descripcin.: Agrupa las funciones de consulta.                                                                       *
*************************************************************************************************************************/

CREATE OR REPLACE PACKAGE UOC.PK_CONSULTAS AS

FUNCTION f_Consulta_A (

  I_Anyo IN NUMBER,
  I_Mes  IN NUMBER
  
  ) RETURN Tabla_Consulta_A;
  
FUNCTION f_Consulta_B (

  I_Anyo IN NUMBER,
  I_Mes  IN NUMBER
  
  ) RETURN Tabla_Consulta_B;
  
FUNCTION f_Consulta_C (

  I_Anyo IN NUMBER,
  I_Mes  IN NUMBER
  
  ) RETURN Tabla_Consulta_C;

FUNCTION f_Consulta_H (

  I_Anyo IN NUMBER
  
  ) RETURN Tabla_Consulta_H;
  
END PK_CONSULTAS;

/

CREATE OR REPLACE PACKAGE BODY UOC.PK_CONSULTAS AS

/************************************************************************************************************************
*                                                                                                                       *  
* Nombre........: f_Consulta_A                                                                                          *
* Descripcin...: Lista todas las tiendas de la cadena incluyendo para cada una:                                        *
*                     1.- El nmero total de productos vendidos durante el mes.                                         *
*                     2.- El nmero de productos diferentes vendidos durante el mes.                                    *
*                     3.- El beneficio neto total obtenido durante el mes.                                              *
*                     4.- El porcentaje de beneficio que aporta la tienda en relacin al total de la cadena durante     *
*                         el mes.                                                                                       *
*                     5.- El beneficio neto dividido por el nmero de empleados de la tienda                            * 
*                                                                                                                       *
* Parmetros....: INPUT..: I_Anyo(Identifica el ao a consultar) - NUMBER(4)        - Obligatorio                       *
*                          I_Mes (Identifica el mes a consultar) - NUMBER(2)        - Obligatorio                       *
*                 OUTPUT.:       (Resultado de la ejecucin    ) - Tabla_Consulta_B                                     *   
*                                                                                                                       *
* Resultado de la ejecucin:                                                                                            *
*                                                                                                                       *
*                 OK - Retorna una tabla con el resultado de la consulta.                                               *
*                 ERROR: No se informa el ao a consultar.                                                              *
*                 ERROR: No se informa el mes a consultar.                                                              *
*                 ERROR: El ao recibido como parmetro no es correcto.                                                 *
*                 ERROR: El mes recibido como parmetro no es correcto.                                                  *
*                 ERROR: SQLCODE - SQLERRM                                                                              *
*                                                                                                                       *
*************************************************************************************************************************/

FUNCTION f_Consulta_A (

  I_Anyo IN NUMBER,
  I_Mes  IN NUMBER
  
  )

RETURN Tabla_Consulta_A

IS

  Datos_Consulta Tabla_Consulta_A := Tabla_Consulta_A();
  
  N_Registro         NUMBER;
  O_RSP              VARCHAR2(250);

  Funcion            VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);

  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);

  C_Tienda                 NUMBER;
  C_Productos_Vendidos     NUMBER;
  C_Productos_Diferentes   NUMBER;
  C_Beneficio_Neto         NUMBER;
  C_Beneficio_Neto_Cadena  NUMBER;
  C_Trabajadores           NUMBER;  
  
  CURSOR c_consulta_a IS
    SELECT 
		H.I_TIENDA,
		SUM(H.I_CANTIDAD_VENTAS)     AS Total_Productos_Vendidos,
		COUNT(DISTINCT H.N_PRODUCTO) AS Numero_Productos_Diferentes,
		SUM(H.N_BENEFICIO_NETO)      AS Total_Beneficio_Neto,
		(SELECT 
			SUM(N_BENEFICIO_NETO) 
		 FROM 
			HECHO 
		 WHERE
			I_TIEMPO IN (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes)
		) AS Total_Beneficio_Neto_Cadena,
		T.N_TRABAJADORES
	FROM 
		HECHO H, TIENDA T 
	WHERE 
		H.I_TIEMPO IN (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes) AND
		H.I_TIENDA = T.I_TIENDA
	GROUP BY H.I_TIENDA, T.N_TRABAJADORES
	ORDER BY Total_Beneficio_Neto DESC;

  PRAGMA AUTONOMOUS_TRANSACTION;
  
BEGIN

  -- Inicializacin de variables
  
  Funcion             := 'PACKAGE.CONSULTAS.f_Consulta_A';
  Parametros_entrada  := '(' || I_Anyo || ', ' || I_Mes || ', ' || ')';
  O_RSP               := 'OK';
  N_Registro          := 0;

  -- Verificacin de datos
 
  PK_VERIFICACION.usp_verificar_anyo(I_Anyo, O_RSP);
  
  IF (O_RSP = 'OK') THEN 
    PK_VERIFICACION.usp_verificar_mes(I_Mes, O_RSP);
  END IF;
  
  IF (O_RSP = 'OK') THEN
  
    OPEN c_consulta_a;
			
      LOOP
      
        FETCH c_consulta_a INTO C_Tienda, C_Productos_Vendidos, C_Productos_Diferentes, C_Beneficio_Neto, C_Beneficio_Neto_Cadena, C_Trabajadores;
				EXIT WHEN c_consulta_a%NOTFOUND;
        
        Datos_consulta.EXTEND;
        N_Registro := N_Registro + 1; 
        Datos_consulta(N_Registro) := obj_consulta_a(
		    C_Tienda,
			C_Productos_Vendidos, 
			C_Productos_Diferentes, 
			C_Beneficio_Neto, 
			((C_Beneficio_Neto * 100) / C_Beneficio_Neto_Cadena),
			(C_Beneficio_Neto / C_Trabajadores));
        
      END LOOP;
      
      CLOSE c_consulta_a;

  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);
  
  Return Datos_consulta;
  
  -- Excepciones 
  
  EXCEPTION

    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);

END f_Consulta_A;

/************************************************************************************************************************
*                                                                                                                       *  
* Nombre........: f_Consulta_B                                                                                          *
* Descripcin...: Lista todas los productos del catalogon incluyendo para cada uno de ellos:                            *
*                     1.- Identificador EAN13.                                                                          *
*                     2.- Nombre del producto.                                                                          *
*                     3.- Nmero de unidades vendidas.                                                                  *
*                     4.- Beneficio neto que ha generado el producto.                                                   *
*                     5.- Tienda que ha vendido ms unidades.                                                           * 
*                     6.- Nmero de unidades vendidas en dicha tienda.                                                  *
*                                                                                                                       *
* Parmetros....: INPUT..: I_Anyo(Identifica el ao a consultar) - NUMBER(4)        - Obligatorio                       *
*                          I_Mes (Identifica el mes a consultar) - NUMBER(2)        - Obligatorio                       *
*                 OUTPUT.:       (Resultado de la ejecucin    ) - Tabla_Consulta_B                                     *   
*                                                                                                                       *
* Resultado de la ejecucin:                                                                                            *
*                                                                                                                       *
*                 OK - Retorna una tabla con el resultado de la consulta.                                               *
*                 ERROR: No se informa el ao a consultar.                                                              *
*                 ERROR: No se informa el mes a consultar.                                                              *
*                 ERROR: El ao recibido como parmetro no es correcto.                                                 *
*                 ERROR: El mes recibido como parmetro no es correcto.                                                  *
*                 ERROR: SQLCODE - SQLERRM                                                                              *
*                                                                                                                       *
*************************************************************************************************************************/

FUNCTION f_Consulta_B (

  I_Anyo IN NUMBER,
  I_Mes  IN NUMBER
  
  )

RETURN Tabla_Consulta_B

IS

  Datos_Consulta Tabla_Consulta_B := Tabla_Consulta_B();
  
  N_Registro         NUMBER;
  O_RSP              VARCHAR2(250);

  Funcion            VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);

  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);

  C_Identificador_Producto   NUMBER;
  C_Nombre_Producto          VARCHAR2(50 CHAR);
  C_Unidades_Vendidas        NUMBER;
  C_Beneficio_Neto           NUMBER;
  C_Tienda                   NUMBER;
  C_Tienda_Unidades_Vendidas NUMBER;  
  
  CURSOR c_consulta_b IS
    SELECT 
		P.N_PRODUCTO,
		P.V_DESCRIPCION,
		SUM(H.I_CANTIDAD_VENTAS) AS Total_Productos_Vendidos,
		SUM(H.N_BENEFICIO_NETO)  AS Total_Beneficio_Neto,
		(SELECT I_TIENDA
			FROM 
				(SELECT I_TIENDA FROM HECHO  
				WHERE I_TIEMPO IN (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes)
				GROUP BY I_Tienda
				ORDER BY Sum(I_Cantidad_Ventas) DESC)
			WHERE ROWNUM <= 1
		) AS Tienda_Mas_Unidades,      
		(SELECT *
			FROM 
				(SELECT SUM(I_CANTIDAD_VENTAS) FROM HECHO 
				WHERE I_TIEMPO IN (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes)
				GROUP BY I_Tienda
				ORDER BY Sum(I_Cantidad_Ventas) DESC) 
			WHERE ROWNUM <= 1
		) AS Tienda_Unidades_Vendidas      
	FROM HECHO H LEFT JOIN PRODUCTO P ON H.N_PRODUCTO = P.N_PRODUCTO
	WHERE H.I_TIEMPO IN (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes) 
	GROUP BY P.N_PRODUCTO, P.V_DESCRIPCION
	ORDER BY Total_Beneficio_Neto DESC;

  PRAGMA AUTONOMOUS_TRANSACTION;
  
BEGIN

  -- Inicializacin de variables
  
  Funcion             := 'PACKAGE.CONSULTAS.f_Consulta_B';
  Parametros_entrada  := '(' || I_Anyo || ', ' || I_Mes || ', ' || ')';
  O_RSP               := 'OK';
  N_Registro          := 0;

  -- Verificacin de datos
 
  PK_VERIFICACION.usp_verificar_anyo(I_Anyo, O_RSP);
  
  IF (O_RSP = 'OK') THEN 
    PK_VERIFICACION.usp_verificar_mes(I_Mes, O_RSP);
  END IF;
  
  IF (O_RSP = 'OK') THEN
  
    OPEN c_consulta_b;
			
      LOOP
        
        FETCH c_consulta_b INTO C_Identificador_Producto, C_Nombre_Producto, C_Unidades_Vendidas, C_Beneficio_Neto, C_Tienda, C_Tienda_Unidades_Vendidas;
				EXIT WHEN c_consulta_b%NOTFOUND;
        
        Datos_consulta.EXTEND;
        N_Registro := N_Registro + 1; 
        Datos_consulta(N_Registro) := obj_consulta_b(
			C_Identificador_Producto, 
			C_Nombre_Producto, 
			C_Unidades_Vendidas, 
			C_Beneficio_Neto, 
			C_Tienda, 
			C_Tienda_Unidades_Vendidas);
        
      END LOOP;
      
      CLOSE c_consulta_b;

  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);
  
  Return Datos_consulta;
  
  -- Excepciones 
  
  EXCEPTION

    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);

END f_Consulta_B;

/************************************************************************************************************************
*                                                                                                                       *  
* Nombre........: f_Consulta_C                                                                                          *
* Descripcin...: Lista todas los das del mes, incluyendo para cada uno de ellos:                                      *
*                     1.- Total beneficio neto obtenido ese da por toda la cadena                                      *
*                     2.- Identificador EAN13 del producto ms vendido                                                  *
*                     3.- Nmero de unidades vendidas del producto ms vendido.                                         *
*                     4.- Identificador de la tienda que ms beneficio neto ha generado.                                *
*                     5.- El valor en euros del beneficio neto generado por dicha tienda.                               * 
*                                                                                                                       *
* Parmetros....: INPUT..: I_Anyo(Identifica el ao a consultar) - NUMBER(4)        - Obligatorio                       *
*                          I_Mes (Identifica el mes a consultar) - NUMBER(2)        - Obligatorio                       *
*                 OUTPUT.:       (Resultado de la ejecucin    ) - Tabla_Consulta_C                                     *   
*                                                                                                                       *
* Resultado de la ejecucin:                                                                                            *
*                                                                                                                       *
*                 OK - Retorna una tabla con el resultado de la consulta.                                               *
*                 ERROR: No se informa el ao a consultar.                                                              *
*                 ERROR: No se informa el mes a consultar.                                                              *
*                 ERROR: El ao recibido como parmetro no es correcto.                                                 *
*                 ERROR: El mes recibido como parmetro no es correcto.                                                  *
*                 ERROR: SQLCODE - SQLERRM                                                                              *
*                                                                                                                       *
*************************************************************************************************************************/

FUNCTION f_Consulta_C (

  I_Anyo IN NUMBER,
  I_Mes  IN NUMBER
  
  )

RETURN Tabla_Consulta_C

IS

  Datos_Consulta Tabla_Consulta_C := Tabla_Consulta_C();
  
  N_Registro         NUMBER;
  O_RSP              VARCHAR2(250);

  Funcion            VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);

  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);

  C_Dia                   NUMBER;
  C_Total_Beneficio_neto  NUMBER;
  C_Producto              NUMBER;
  C_Cantidad_Ventas       NUMBER;
  C_Tienda                NUMBER;
  C_Tienda_Beneficio_Neto NUMBER;  
  
  CURSOR c_consulta_c IS
		select A.N_DIA, B.Total_Beneficio_Neto, C.N_Producto, C.Cantidad_Ventas, D.I_tienda, D.Tienda_Beneficio_Neto from 
			(select i_tiempo, n_dia from tiempo 
				where i_tiempo in (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes) 
				order by n_dia) A 
		left join 
			(select i_tiempo, sum(n_beneficio_neto) AS Total_Beneficio_Neto from  hecho  
				where i_tiempo in (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes)
				group by i_tiempo
				order by Total_Beneficio_Neto desc) B ON A.i_tiempo = B.i_tiempo
		left join
			(select i_tiempo, n_producto, sum(i_cantidad_ventas) AS Cantidad_Ventas from hecho
				where i_tiempo in (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes)
				group by i_tiempo, n_producto
				order by Cantidad_Ventas desc) C ON A.I_TIEMPO = C.I_TIEMPO
		left join 
			(select i_tiempo, i_tienda, sum(n_beneficio_neto) AS Tienda_Beneficio_Neto from hecho 
			where i_tiempo in (SELECT I_Tiempo FROM Tiempo where N_ANYO = I_Anyo and N_MES = I_Mes)
			group by i_tiempo, i_tienda
			order by Tienda_Beneficio_Neto desc) D ON A.i_tiempo = d.i_tiempo;

  PRAGMA AUTONOMOUS_TRANSACTION;
  
BEGIN

  -- Inicializacin de variables
  
  Funcion             := 'PACKAGE.CONSULTAS.f_Consulta_C';
  Parametros_entrada  := '(' || I_Anyo || ', ' || I_Mes || ', ' || ')';
  O_RSP               := 'OK';
  N_Registro          := 0;

  -- Verificacin de datos
 
  PK_VERIFICACION.usp_verificar_anyo(I_Anyo, O_RSP);
  
  IF (O_RSP = 'OK') THEN 
    PK_VERIFICACION.usp_verificar_mes(I_Mes, O_RSP);
  END IF;
  
  IF (O_RSP = 'OK') THEN
  
    OPEN c_consulta_c;
			
      LOOP
  
        FETCH c_consulta_c INTO C_Dia, C_Total_Beneficio_Neto, C_Producto, C_Cantidad_Ventas, C_Tienda, C_Tienda_Beneficio_Neto;
				EXIT WHEN c_consulta_c%NOTFOUND;
        
        Datos_consulta.EXTEND;
        N_Registro := N_Registro + 1; 
        Datos_consulta(N_Registro) := obj_consulta_c(
			C_Dia, 
			C_Total_Beneficio_Neto, 
			C_Producto, 
			C_Cantidad_Ventas, 
			C_Tienda, 
			C_Tienda_Beneficio_Neto);
        
      END LOOP;
      
      CLOSE c_consulta_c;

  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);
  
  Return Datos_consulta;
  
  -- Excepciones 
  
  EXCEPTION

    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);

END f_Consulta_C;

/************************************************************************************************************************
*                                                                                                                       *  
* Nombre........: f_Consulta_H                                                                                          *
* Descripcin...: Lista la informacin asociada al histrico para un ao o para todos:                                  *
*                     1.- Beneficio neto de toda la cadena.                                                             *
*                     2.- Identificador de la tienda que ms beneficio neto ha conseguido y la cantidad conseguida.     *
*                     3.- Identificador del producto ms vendido y el nmero de unidades vendidas.                      *
*                     4.- La hora del da donde ms productos se han vendido y el nmero de unidades vendidas.          *
*                     5.- La hora del da donde menos productos se han vendido y el nmero de unidades vendidas.        *
*                     6.- El da del mes donde ms productos se han vendido y el nmero de unidades vendidas.           *
*                     7.- El da del mes donde menos productos se han vendido y el nmero de unidades vendidas.         *
*                     8.- Ciudad donde ms beneficios se han conseguido y la cantidad conseguida.                       * 
*                     5.- Porcentaje de beneficios obtenidos por las tiendas virtuales con respecto a la cadena.        * 
*                                                                                                                       *
* Parmetros....: INPUT..: I_Anyo(Identifica el ao a consultar) - NUMBER(4)                                            *
*                 OUTPUT.:       (Resultado de la ejecucin    ) - Tabla_Consulta_H                                     *   
*                                                                                                                       *
* Resultado de la ejecucin:                                                                                            *
*                                                                                                                       *
*                 OK - Retorna una tabla con el resultado de la consulta.                                               *
*                 ERROR: El ao recibido como parmetro no es correcto.                                                 *
*                 ERROR: SQLCODE - SQLERRM                                                                              *
*                                                                                                                       *
*************************************************************************************************************************/

FUNCTION f_Consulta_H (

  I_Anyo IN NUMBER
  
  )

RETURN Tabla_Consulta_H

IS

  Datos_Consulta Tabla_Consulta_H := Tabla_Consulta_H();
  
  N_Registro         NUMBER;
  O_RSP              VARCHAR2(250);

  Funcion            VARCHAR2(50 CHAR);
  Parametros_entrada VARCHAR2(1000 CHAR);

  Err_Codigo         NUMBER;
  Err_Descripcion    VARCHAR2(200 CHAR);

  C_ANYO                NUMBER;
  C_BNETO_CADENA        NUMBER;
  C_TIENDA              INTEGER;
  C_BNETO_TIENDA        NUMBER;
  C_PRODUCTO            NUMBER;
  C_PRODUCTO_UNIDADES   INTEGER;
  C_HORA_MAS            NUMBER;
  C_HORA_MAS_UNIDADES   INTEGER;
  C_HORA_MENOS          NUMBER;
  C_HORA_MENOS_UNIDADES INTEGER;
  C_DIA_MAS             NUMBER;
  C_DIA_MAS_UNIDADES    INTEGER;
  C_DIA_MENOS           NUMBER;
  C_DIA_MENOS_UNIDADES  INTEGER;
  C_CIUDAD              INTEGER;
  C_BNETO_CIUDAD        NUMBER;
  C_PB_VIRTUALES        NUMBER;
  
  CURSOR c_consulta_h IS
		select N_ANYO, N_BNETO_CADENA, I_TIENDA, N_BNETO_TIENDA, N_PRODUCTO, I_PRODUCTO_UNIDADES, N_HORA_MAS, I_HORA_MAS_UNIDADES,
		       N_HORA_MENOS, I_HORA_MENOS_UNIDADES, N_DIA_MAS, I_DIA_MAS_UNIDADES, N_DIA_MENOS, I_DIA_MENOS_UNIDADES, I_CIUDAD,
			   N_BNETO_CIUDAD, N_PB_VIRTUALES from estadistico where n_anyo = nvl(I_Anyo,0);
		
  PRAGMA AUTONOMOUS_TRANSACTION;
  
BEGIN

  -- Inicializacin de variables
  
  Funcion             := 'PACKAGE.CONSULTAS.f_Consulta_H';
  Parametros_entrada  := '(' || I_Anyo || ')';
  O_RSP               := 'OK';
  N_Registro          := 0;

  -- Verificacin de datos
 
  IF (NVL(I_Anyo,0) <> 0) THEN
     PK_VERIFICACION.usp_verificar_anyo(I_Anyo, O_RSP);
  END IF;
  
  IF (O_RSP = 'OK') THEN
  
    OPEN c_consulta_h;
			
      LOOP
  
        FETCH c_consulta_h INTO C_ANYO, C_BNETO_CADENA, C_TIENDA, C_BNETO_TIENDA, C_PRODUCTO, C_PRODUCTO_UNIDADES, C_HORA_MAS, C_HORA_MAS_UNIDADES,
								C_HORA_MENOS, C_HORA_MENOS_UNIDADES, C_DIA_MAS, C_DIA_MAS_UNIDADES, C_DIA_MENOS, C_DIA_MENOS_UNIDADES, C_CIUDAD,
								C_BNETO_CIUDAD, C_PB_VIRTUALES;
				EXIT WHEN c_consulta_h%NOTFOUND;
        
        Datos_consulta.EXTEND;
        N_Registro := N_Registro + 1; 
        Datos_consulta(N_Registro) := obj_consulta_h(
			C_ANYO, 
			C_BNETO_CADENA, 
			C_TIENDA, 
			C_BNETO_TIENDA, 
			C_PRODUCTO, 
			C_PRODUCTO_UNIDADES, 
			C_HORA_MAS,
		    C_HORA_MAS_UNIDADES, 
			C_HORA_MENOS, 
			C_HORA_MENOS_UNIDADES, 
			C_DIA_MAS, 
			C_DIA_MAS_UNIDADES, 
			C_DIA_MENOS,
            C_DIA_MENOS_UNIDADES, 
			C_CIUDAD, 
			C_BNETO_CIUDAD, 
			C_PB_VIRTUALES);
        
      END LOOP;
      
      CLOSE c_consulta_h;

  END IF;
  
  -- Registra la ejecucin en el log
  
  PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);
  
  Return Datos_consulta;
  
  -- Excepciones 
  
  EXCEPTION

    WHEN OTHERS THEN
      Err_Codigo      := SQLCODE;
      Err_Descripcion := SQLERRM;
      
      O_RSP           := 'ERROR: ' || TO_CHAR(Err_Codigo) 
                                   || '-' 
                                   || Err_Descripcion;
      
      PK_PROCEDIMIENTOS.usp_insertar_log(Funcion, Parametros_entrada, O_RSP);

END f_Consulta_H;

END PK_CONSULTAS;

/

SPOOL OFF;
